# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: 
"""
import os

from companynameparser import parser


def predict_tags(input_list, predict_file):
    model = parser.Parser()
    with open(predict_file, 'w', encoding='utf-8') as f:
        for line in input_list:
            r = model.parse(line, pos_sensitive=False)
            b = r['brand']
            f.write(line + '\t' + b + '\n')


def load_file(file_path, limit_size=20):
    subs = []
    count = 0
    if not os.path.exists(file_path):
        return subs
    with open(file_path, 'r', encoding='utf-8') as fr:
        for line in fr:
            i = line.strip().split()[0]
            if i:
                count += 1
                subs.append(i)
            if 0 < limit_size <= count:
                break
    return subs


def horizontal_bio(predict_result_file, horizontal_file):
    with open(predict_result_file, 'r', encoding='utf-8') as fr, \
            open(horizontal_file, 'w', encoding='utf-8') as fw:
        for line in fr:
            terms = line.split()
            i = terms[0]
            b = terms[1].strip().split(',')[0] if len(terms) == 2 else ''
            if b:
                brand_idx = i.index(b)
                if brand_idx > -1:
                    brand_start = brand_idx
                    brand_len = len(b)
                    if brand_len == 1:
                        continue
                    out = ' '.join(['O'] * brand_start + ['B-ORG'] +
                                   ['I-ORG'] * (brand_len - 1) + ['O'] * (
                            len(i) - brand_start - brand_len))
            else:
                out = ' '.join(['O'] * len(i))
            fw.write(i + '\t' + out + '\n')
            print(i + '\t' + out)


def vertical_bio(horizontal_file, out_vertical_file):
    with open(horizontal_file, 'r', encoding='utf-8') as fr, \
            open(out_vertical_file, 'w', encoding='utf-8') as fw:
        for line in fr:
            line = line.strip()
            terms = line.split('\t')
            chars = list(terms[0])
            tags = terms[1].split(' ')
            if len(chars) != len(tags):
                continue
            for i in range(len(chars)):
                fw.write(chars[i] + '\t' + tags[i] + '\n')
            fw.write('\n')


def main():
    domain_name_file = '../data/company_demo.txt'
    # predict_file format: sentence '\t' brand1,brand2
    predict_file = 'sentence_brands.txt'
    horizontal_file = 'hor_train.txt'
    vertical_file = 'ver_train.txt'
    c = load_file(domain_name_file, limit_size=10)
    # predict, output brand name
    predict_tags(c, predict_file)
    # to bio
    horizontal_bio(predict_file, horizontal_file)
    # to vertical bio
    vertical_bio(horizontal_file, vertical_file)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: 
"""

import os

pwd_path = os.path.abspath(os.path.dirname(__file__))
# 区划地址文件
pca_path = os.path.join(pwd_path, 'data/pca.csv')


def get_places():
    """
    地名 -> 地名全称
    :return: dict
    """
    place_map = {}
    # 数据约定:国家直辖市的sheng字段为直辖市名称, 省直辖县的city字段为空

    with open(pca_path, 'r', encoding='utf-8') as f:
        import csv
        pca_csv = csv.DictReader(f)
        for record_dict in pca_csv:
            _fill_place_map(place_map, record_dict)
    return place_map


def _fill_place_map(place_map, record_dict):
    """
    填充地名，包括简称
    :param place_map: dict
    :param record_dict: dict
    :return: place_map
    """
    sheng = record_dict['sheng']
    if sheng not in place_map:
        place_map[sheng] = sheng
        # 处理省的简写情况
        # 普通省分 和 直辖市
        if sheng.endswith('省') or sheng.endswith('市'):
            place_map[sheng[:-1]] = sheng
        # 自治区
        elif sheng == '新疆维吾尔自治区':
            place_map['新疆'] = sheng
        elif sheng == '内蒙古自治区':
            place_map['内蒙古'] = sheng
        elif sheng == '广西壮族自治区':
            place_map['广西'] = sheng
            place_map['广西省'] = sheng
        elif sheng == '西藏自治区':
            place_map['西藏'] = sheng
        elif sheng == '宁夏回族自治区':
            place_map['宁夏'] = sheng
        # 特别行政区
        elif sheng == '香港特别行政区':
            place_map['香港'] = sheng
        elif sheng == '澳门特别行政区':
            place_map['澳门'] = sheng

    city_name = record_dict['shi']
    if city_name not in place_map:
        place_map[city_name] = city_name
        # 处理简写情况
        if city_name.endswith('市'):
            place_map[city_name[:-1]] = city_name
        # 特别行政区
        elif city_name == '香港特别行政区':
            place_map['香港'] = city_name
        elif city_name == '澳门特别行政区':
            place_map['澳门'] = city_name
        # 自治区下的二级区划，eg喀什地区
        elif len(city_name) > 3 and city_name.endswith('地区'):
            place_map[city_name[:-2]] = city_name

    area_name = record_dict['qu']
    if area_name not in place_map:
        place_map[area_name] = area_name
        # 处理简写情况
        # 4字区划简称
        if len(area_name) > 3 and (area_name.endswith('新区') or area_name.endswith('城区') or area_name.endswith('林区')):
            place_map[area_name[:-2]] = area_name
        # 3字区划简称，'XX区'不简写
        elif len(area_name) > 2 and (area_name.endswith('市') or area_name.endswith('县') or area_name.endswith('区')):
            place_map[area_name[:-1]] = area_name


if __name__ == '__main__':
    maps = get_places()
    with open('place.txt', 'w', encoding='utf-8') as f:
        for k, v in maps.items():
            f.write(k + '\n')



# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: 配置切词器
"""

import jieba

jieba.setLogLevel(log_level="ERROR")


def jieba_tokenize(sentence):
    """
    jieba分词
    """
    return list(jieba.tokenize(sentence))


def hanlp_tokenize(sentence):
    """
    hanlp分词
    """
    import os
    import hanlp

    os.environ["KMP_DUPLICATE_LIB_OK"] = "TRUE"
    hanlp_parser = hanlp.load(hanlp.pretrained.mtl.CLOSE_TOK_POS_NER_SRL_DEP_SDP_CON_ELECTRA_SMALL_ZH)  # 中文语料库
    sent_list = hanlp_parser(sentence, tasks='tok/fine').get('tok/fine')
    res = []
    start_idx = 0
    for word in sent_list:
        end_idx = start_idx + len(word)
        res.append((word, start_idx, end_idx))
        start_idx = end_idx
    return res


if __name__ == '__main__':
    text = "阿联酋阿斯迪拉电子贸易公司"
    print(jieba_tokenize(text))
    print(hanlp_tokenize(text))



# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: 
"""

from companynameparser.parser import Parser

__version__ = "0.1.8"

par = Parser()
parse = par.parse
set_custom_split_file = par.set_custom_split_file



# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: 
"""

import os
import time
from companynameparser.logger import logger
from companynameparser.tokenizer import jieba_tokenize

pwd_path = os.path.abspath(os.path.dirname(__file__))
# 地址文件
place_path = os.path.join(pwd_path, 'data/china_place.txt')
# 地址补充文件，单字类
place_single_path = os.path.join(pwd_path, 'data/place_single.txt')
# 商标文件
brand_path = os.path.join(pwd_path, 'data/brand.txt')
# 行业文件
trade_path = os.path.join(pwd_path, 'data/trade.txt')
# 行业补充文件，单字类
trade_single_path = os.path.join(pwd_path, 'data/trade_single.txt')
# 后缀文件
suffix_path = os.path.join(pwd_path, 'data/suffix.txt')
# 后缀补充文件，单字类
suffix_single_path = os.path.join(pwd_path, 'data/suffix_single.txt')
# 词语分隔符，split word by comma
split_sep = ','


class Parser:
    """
    Name Parser for Company Name
    """

    def __init__(
            self,
            place_file=place_path,
            brand_file=brand_path,
            trade_file=trade_path,
            suffix_file=suffix_path,
            place_single_file=place_single_path,
            trade_single_file=trade_single_path,
            suffix_single_file=suffix_single_path,
            custom_name_split_file='',
    ):
        self.name = 'company_name_parser'
        self.place_file = place_file
        self.brand_file = brand_file
        self.trade_file = trade_file
        self.suffix_file = suffix_file
        self.place_single_file = place_single_file
        self.trade_single_file = trade_single_file
        self.suffix_single_file = suffix_single_file
        self.custom_name_split_file = custom_name_split_file
        self.places = None
        self.brands = None
        self.trades = None
        self.suffixes = None
        self.place_single = None
        self.trade_single = None
        self.suffix_single = None
        self.custom_name_split = None
        self.symbols = ['《', '》', '（', '）', '(', ')']
        self.inited = False

    def init(self):
        if not self.inited:
            s = time.time()
            self.places = self.load_dict(self.place_file)
            self.brands = self.load_dict(self.brand_file)
            self.trades = self.load_dict(self.trade_file)
            self.suffixes = self.load_dict(self.suffix_file)
            self.place_single = self.load_dict(self.place_single_file)
            self.trade_single = self.load_dict(self.trade_single_file)
            self.suffix_single = self.load_dict(self.suffix_single_file)
            self.custom_name_split = self.load_name_split(self.custom_name_split_file)
            self.inited = True
            # logger.debug('dict load ok, cost {:.2f} s.'.format(time.time() - s))

    def set_custom_split_file(self, file_path):
        self.custom_name_split_file = file_path
        self.custom_name_split = self.load_name_split(self.custom_name_split_file)
        logger.debug('set_custom_split_file done, custom_name_split size: {}'.format(len(self.custom_name_split)))

    @staticmethod
    def load_dict(file_path):
        res = dict()
        if file_path and os.path.exists(file_path):
            with open(file_path, 'r', encoding='utf-8') as f:
                for line in f:
                    line = line.strip()
                    terms = line.split()
                    if len(terms) == 2:
                        key = terms[0]
                        val = terms[1]
                    elif len(terms) == 1:
                        key = terms[0]
                        val = '1'
                    else:
                        continue
                    res[key] = val
        return res

    @staticmethod
    def load_name_split(file_path):
        res = dict()
        if file_path and os.path.exists(file_path):
            with open(file_path, 'r', encoding='utf-8') as f:
                for line in f:
                    line = line.strip()
                    terms = line.split(',')
                    if line.startswith('#') or len(terms) <= 1:
                        continue
                    # Company_name place brand trade suffix symbol
                    name = terms[0]
                    place = terms[1] if len(terms) >= 2 else ''
                    brand = terms[2] if len(terms) >= 3 else ''
                    trade = terms[3] if len(terms) >= 4 else ''
                    suffix = terms[4] if len(terms) >= 5 else ''
                    symbol = terms[5] if len(terms) >= 6 else ''
                    res[name] = {'place': place,
                                 'brand': brand,
                                 'trade': trade,
                                 'suffix': suffix,
                                 'symbol': symbol}
        return res

    @staticmethod
    def findall(string, s):
        """
        Find all str index
        :param string:
        :param s:
        :return: indies
        """
        res = []
        index = 0
        while True:
            index = string.find(s, index)
            if index != -1:
                res.append(index)
                index += len(s)
            else:
                break
        return res

    @staticmethod
    def _extract_token(tokens, data_dict):
        """
        Extract token words
        :param tokens: query segmented words
        :param data_dict: dict
        :return: tuple(token, left_words)
        """
        res = []
        left_words = []
        for w, p, q in tokens:
            if w in data_dict:
                res.append((w, p, q))
            else:
                left_words.append((w, p, q))
        return res, left_words

    @staticmethod
    def link_near_words(tokens):
        new_tokens = []
        if not tokens:
            return new_tokens
        i = 0
        w, p, q = tokens[i]
        while i < len(tokens):
            i += 1
            if i == len(tokens):
                new_tokens.append((w, p, q))
            else:
                w_i, p_i, q_i = tokens[i]
                # next word near before word
                if p_i == q:
                    w = w + w_i
                    p = p
                    q = q_i
                else:
                    new_tokens.append((w, p, q))
                    w, p, q = tokens[i]
        return new_tokens

    def postprocess(self, left_words, places, brands, trades, suffixes):
        """
        Extract Company name brand words
        :param left_words: query segmented words
        :param places: places
        :param brands: brands
        :param trades: trades
        :param suffixes: suffixes
        :return: tuple(places, brands, trades, suffixes)
        """
        lefts = []
        for w, p, q in left_words:
            if len(w) == 1:
                # Single char
                if w in self.trade_single:
                    if len(left_words) == 2 and len(left_words[0][0]) == 1 and len(
                            left_words[1][0]) == 1 and not brands:
                        # 处理："天津曰新塑料制品有限公司"
                        brands.append((w, p, q))
                    else:
                        trades.append((w, p, q))
                elif w in self.place_single:
                    places.append((w, p, q))
                elif w in self.suffix_single:
                    suffixes.append((w, p, q))
                else:
                    # Left single char, which brand not single char
                    lefts.append((w, p, q))
            else:
                if w[-1] in self.place_single:
                    places.append((w, p, q))
                else:
                    brands.append((w, p, q))
        if len(lefts) == 1:
            if len(places) > 1 and places[0][2] == lefts[0][1] and places[1][1] == lefts[0][2]:
                # 处理："永安市燕南街扶晴梅百货商行"
                places.extend(lefts)
            elif len(trades) > 1 and lefts[0][2] == trades[0][1] and not brands:
                # 处理："中节能秦皇岛环保有限公司"
                brands.extend(lefts)
                brands.append(trades[0])
                trades.remove(trades[0])
            else:
                brands.extend(lefts)
        else:
            brands.extend(lefts)
        if len(places) > 1:
            places.sort(key=lambda k: k[1])
            places = self.link_near_words(places)  # Deal with link near words
        if len(brands) > 1:
            brands.sort(key=lambda k: k[1])
            brands = self.link_near_words(brands)
        return places, brands, trades, suffixes

    @staticmethod
    def _get_leave_tokens(tokens, start, end):
        """
        String out of [start, end]
        :param tokens: list, [(word, start_idx, end_idx),...]
        :param start: int, start_idx
        :param end: int, end_idx
        :return: list, < start_idx || > end_idx
        """
        res = []
        for w, p, q in tokens:
            if p < start:
                res.append((w, p, q))
            elif p >= end:
                res.append((w, p, q))
        return res

    def parse(self, name, pos_sensitive=False, enable_word_segment=False, **kwargs):
        """
        Parse One Record
        :param name: Company name
        :param pos_sensitive: if True, output position index; default False
        :param enable_word_segment: if True, output word split by comma, else no comma; default False
        :return: dict
        """
        name = name.strip()
        res = {'place': '', 'brand': '', 'trade': '', 'suffix': '', 'symbol': ''}
        # Not Chinese company name
        if (not name) or (not is_chinese(name[0])):
            return res

        self.init()
        places, brands, trades, suffixes, symbols = [], [], [], [], []
        # Tokens: [(word, start_index, end_index), ...]
        tokens = jieba_tokenize(name)

        if self.custom_name_split:
            # Custom name split
            for k, v in self.custom_name_split.items():
                start = name.find(k)
                if start > -1:
                    end = start + len(k)
                    tokens = self._get_leave_tokens(tokens, start, end)
                    place_len = len(v['place'])
                    brand_len = len(v['brand'])
                    trade_len = len(v['trade'])
                    suffix_len = len(v['suffix'])
                    c_places = [(v['place'], start, start + place_len)] if place_len > 0 else []
                    c_brands = [(v['brand'], start + place_len, start + place_len + brand_len)] if brand_len > 0 else []
                    c_trades = [(v['trade'], start + place_len + brand_len,
                                 start + place_len + brand_len + trade_len)] if trade_len > 0 else []
                    c_suffixes = [(v['suffix'], start + place_len + brand_len + trade_len,
                                   start + place_len + brand_len + trade_len + suffix_len)] if suffix_len > 0 else []
                    places.extend(c_places)
                    brands.extend(c_brands)
                    trades.extend(c_trades)
                    suffixes.extend(c_suffixes)
                    break

        if tokens:
            # Extract token order: symbol, place, suffix, trade, brand
            t_symbols, left_words = self._extract_token(tokens, self.symbols)
            t_places, left_words = self._extract_token(left_words, self.places)
            t_suffixes, left_words = self._extract_token(left_words, self.suffixes)
            t_trades, left_words = self._extract_token(left_words, self.trades)
            t_brands, left_words = self._extract_token(left_words, self.brands)
            # Postprocess
            t_places, t_brands, t_trades, t_suffixes = self.postprocess(left_words, t_places, t_brands, t_trades,
                                                                        t_suffixes)
            places.extend(t_places)
            brands.extend(t_brands)
            trades.extend(t_trades)
            suffixes.extend(t_suffixes)
            symbols.extend(t_symbols)

        # Sort token idx
        if len(places) > 1:
            places.sort(key=lambda k: k[1])
        if len(brands) > 1:
            brands.sort(key=lambda k: k[1])
        if len(trades) > 1:
            trades.sort(key=lambda k: k[1])
        if len(suffixes) > 1:
            suffixes.sort(key=lambda k: k[1])

        # Link split tokens
        if not enable_word_segment:
            places = self.link_near_words(places)
            brands = self.link_near_words(brands)
            trades = self.link_near_words(trades)
            suffixes = self.link_near_words(suffixes)

        # Pos Sensitive Enable
        res['place'] = places if pos_sensitive else split_sep.join([w[0] for w in places])
        res['brand'] = brands if pos_sensitive else split_sep.join([w[0] for w in brands])
        res['trade'] = trades if pos_sensitive else split_sep.join([w[0] for w in trades])
        res['suffix'] = suffixes if pos_sensitive else split_sep.join([w[0] for w in suffixes])
        res['symbol'] = symbols if pos_sensitive else split_sep.join([w[0] for w in symbols])

        return res


def is_chinese(uchar):
    """判断一个unicode是否是汉字"""
    return '\u4e00' <= uchar <= '\u9fa5'


def is_number(uchar):
    """判断一个unicode是否是数字"""
    return '\u0030' <= uchar <= '\u0039'


def is_alphabet(uchar):
    """判断一个unicode是否是英文字母"""
    return '\u0041' <= uchar <= '\u005a' or '\u0061' <= uchar <= '\u007a'


def is_other(uchar):
    """判断是否非汉字，数字和英文字符"""
    return not (is_chinese(uchar) or is_number(uchar) or is_alphabet(uchar))



# -*- coding: utf-8 -*-
"""
@author:XuMing(xuming624@qq.com)
@description: 
"""
import sys

tag_idx = {
    "B-PER": 0,
    "I-PER": 1,
    "B-ORG": 2,
    "I-ORG": 3,
    "B-LOC": 4,
    "I-LOC": 5,
    "O": 6,
}
idx_tag = {idx: tag for tag, idx in tag_idx.items()}


def ids_2_tags(ids):
    return [idx_tag[idx] for idx in ids]


def get_entity_from_bio(chars, tags):
    """
    从BIO结果中获取brand
    """
    chunks = []
    current = None
    if len(chars) != len(tags):
        return chunks
    for i, tag in enumerate(tags):
        if tag == 'B-ORG':
            if current is not None and len(current) > 1:
                chunks.append(''.join(current))
            current = [chars[i]]
        elif tag == 'I-ORG':
            if current is not None:
                current.append(chars[i])
        elif tag == 'O':
            if current is not None and len(current) > 1:
                chunks.append(''.join(current))
            current = None
    if current is not None and len(current) > 1:
        chunks.append(''.join(current))
    return chunks


if __name__ == "__main__":
    for line in sys.stdin:
        line = line.strip()
        terms = line.split('\t')
        chars = terms[0]
        tags = terms[1].split(" ")
        chunks = get_entity_from_bio(list(chars), tags)
        print(chars + " " + " ".join(chunks))
